/*
 * AbstractElementFactory.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Collections;
import java.util.Map;
import java.util.Set;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamElementFactory;
import org.jabberstudio.jso.io.StreamBuilder;

/**
 *
 * @author  linuxwolf
 */
public abstract class AbstractElementFactory implements StreamElementFactory {
    //Fields
    private NSI.Comparator  _Cmpr;
    private Map             _Supported;
    
    //Constructors
    public AbstractElementFactory(NSI.Comparator cmpr) throws IllegalArgumentException {
        if (cmpr == null)
            cmpr = NSI.STRICT_COMPARATOR;
        _Cmpr = cmpr;
        _Supported = new java.util.TreeMap(cmpr);
        
        registerSupportedElements();
    }
    
    //Methods
    protected NSI.Comparator getComparator() {
        return _Cmpr;
    }
    
    protected abstract void registerSupportedElements() throws IllegalArgumentException;
    protected NSI adjustNSI(NSI name) throws IllegalArgumentException {
        return name;
    }
    
    public Set getSupportedElements() {
        return Collections.unmodifiableSet(_Supported.keySet());
    }

    protected AbstractElement getSupportedElement(NSI name, Class iface) {
        AbstractElement elem;
        
        name = adjustNSI((name == null) ? new NSI(null, null) : name);
        elem = (AbstractElement)_Supported.get(name);

        if ((elem != null) && (iface != null) && !iface.isInstance(elem))
            elem = null;

        return elem;
    }
    protected void putSupportedElement(AbstractElement elem) {
        if (elem != null)
            putSupportedElement(elem.getNSI(), elem);
    }
    protected void putSupportedElement(NSI name, StreamElement elem) {
        if ((name != null) && (elem != null))
            _Supported.put(name, elem);
    }
    protected void removeSupportedElement(NSI name) {
        if (name != null)
            _Supported.remove(name);
    }
    
    public boolean isSupportedElement(NSI name, Class iface) {
        AbstractElement     elem = getSupportedElement(name, iface);

        return (elem != null);
    }
    public StreamElement createElementNode(NSI name, Class iface, StreamDataFactory sdf) throws IllegalArgumentException, IllegalStateException {
        return createElementBase(name, iface, sdf);
    }
    public StreamBuilder createElementBuilder(NSI name, Class iface, StreamDataFactory sdf) throws IllegalArgumentException, IllegalStateException {
        return createElementBase(name, iface, sdf).createBuilder();
    }
    
    protected StreamElement createElementBase(NSI name, Class iface, StreamDataFactory sdf) throws IllegalArgumentException, IllegalStateException {
        AbstractElement     elem;
        
        if (name == null)
            throw new IllegalArgumentException("NSI cannot be null");
        //name = adjustNSI(name);
        elem = getSupportedElement(name, iface);
        
        if (elem == null)
            throw new IllegalStateException("NSI is not supported");
        elem = (AbstractElement)elem.copy();
        elem.reset(name);
        elem.setDataFactory(sdf);
        
        return elem;
    }
}