/*
 * UnknownFactory.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Collections;
import java.util.Set;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamElementFactory;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Factory to create <code>StreamNode</code> and <code>StreamBuilder<code>
 * objects for <code>UnknownPacket</code> and <code>UnknownExtension</code>.
 * Instances of this factory are typically used for 
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class UnknownFactory implements StreamElementFactory {
    //Fields
    private static Set  _Supported = Collections.singleton(new NSI(null, null));
    
    //Constructors
    public UnknownFactory() {
    }
    
    //Methods
    public Set getSupportedElements() {
        return _Supported;
    }
    public boolean isSupportedElement(NSI name, Class iface) {
        boolean     retval = (iface == null);
        
        if (!retval) {
            retval = (  iface.isAssignableFrom(ExtensionNode.class) ||
                        iface.isAssignableFrom(UnknownPacketNode.class) ||
                        iface.isAssignableFrom(UnknownConditionNode.class) ||
                        iface.isAssignableFrom(UnknownFeatureNode.class) ||
                        iface.isAssignableFrom(ElementNode.class));
        }
        
        return retval;
    }
    public StreamElement createElementNode(NSI name, Class iface, StreamDataFactory sdf) throws IllegalArgumentException {
        StreamElement       elem = null;
        
        if (name == null)
            throw new IllegalArgumentException("NSI cannot be null");
        if (!Utilities.isValidString(name.getLocalName()))
            throw new IllegalArgumentException("Local name cannot be null or \"\"");
        
        if (iface != null) {
            if (iface.isAssignableFrom(UnknownConditionNode.class))     elem = new UnknownConditionNode(sdf, name);
            else if (iface.isAssignableFrom(UnknownFeatureNode.class))  elem = new UnknownFeatureNode(sdf, name);
            else if (iface.isAssignableFrom(ExtensionNode.class))       elem = new ExtensionNode(sdf, name);
            else if (iface.isAssignableFrom(UnknownPacketNode.class))   elem = new UnknownPacketNode(sdf, name);
            else if (iface.isAssignableFrom(ElementNode.class))         elem = new ElementNode(sdf, name);
        } else {
            elem = new ElementNode(sdf, name);
        }
        
        if (elem == null)
            throw new IllegalArgumentException("NSI and interface are not supported");
        
        return elem;
    }
    public StreamBuilder createElementBuilder(NSI name, Class iface, StreamDataFactory sdf) throws IllegalArgumentException {
        return createElementNode(name, iface, sdf).createBuilder();
    }
}