/*
 * DOMWriterImpl.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.io;


import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.jabberstudio.jso.StreamAttribute;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNamespace;
import org.jabberstudio.jso.StreamText;
import org.jabberstudio.jso.io.DOMExporter;
import org.jabberstudio.jso.util.BasicStreamObjectVisitor;
import org.jabberstudio.jso.util.Utilities;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

/**
 *
 * @author  linuxwolf
 */
public class DOMWriter implements DOMExporter {
    //Inner-classes
    private class Visitor extends BasicStreamObjectVisitor {
        //Fields
        Document        _Doc;
        
        Element         _Current;

        //Constructors
        public Visitor(Document doc) {
            _Doc = doc;
        }
        
        //Methods
        public Document getDocument() {
            return _Doc;
        }
        
        public Element getCurrent() {
            return _Current;
        }
        public void setCurrent(Element curr) {
            _Current = curr;
        }
        
        public String generateQName(StreamElement elem) {
            String          local = elem.getLocalName();
            String          prefix = elem.getNamespacePrefix();
            StringBuffer    qname = new StringBuffer();

            if (Utilities.isValidString(prefix))
                qname.append(prefix).append(":");
            qname.append(local);

            return qname.toString();
        }
        public String generateQName(StreamAttribute attr) {
            String          local = attr.getLocalName();
            String          prefix = attr.getNamespacePrefix();
            StringBuffer    qname = new StringBuffer();
            
            if (Utilities.isValidString(prefix))
                qname.append(prefix).append(":");
            qname.append(local);

            return qname.toString();
        }
        
        public void visit(StreamElement in) {
            String      qname = generateQName(in);
            String      uri = in.getNamespaceURI();
            Iterator    nss = in.getDeclaredNamespaces().iterator();
            Element     out = getDocument().createElementNS(uri, qname);
            Element     parent = getCurrent();

            //Setup namespace declarations
            while (nss.hasNext()) {
                StreamNamespace nsObj = (StreamNamespace)nss.next();
                String          nsURI = nsObj.getURI();
                String          nsPrefix = nsObj.getPrefix();
                Attr            nsAttr;
                
                nsAttr = getDocument().createAttribute(!Utilities.isValidString(nsPrefix) ? "xmlns" : "xmlns:" + nsPrefix);
                nsAttr.setValue(nsURI);
                
                out.setAttributeNode(nsAttr);
            }

            //Process "children" (Attribute and Nodes)
            setCurrent(out);
            super.visit(in);
            setCurrent(parent);

            if (parent == null)
                //This is the root
                setCurrent(out);
            else
                //Append to "root"
                parent.appendChild(out);
        }
        
        public void visit(StreamAttribute in) {
            String      qname = generateQName(in);
            String      uri = in.getNamespaceURI();
            Element     out = getCurrent();
            Attr        attr = getDocument().createAttributeNS(uri, qname);

            attr.setValue(in.getValue());
            out.setAttributeNodeNS(attr);
        }
        
        public void visit(StreamText in) {
            Element     out = getCurrent();
            Text        txt = getDocument().createTextNode(in.getValue());
            
            out.appendChild(txt);
        }
    }
    
    //Fields
    private StreamElement   _Ctx;
    
    //Constructors
    public DOMWriter() {
    }
    public DOMWriter(StreamElement ctx) throws IllegalArgumentException {
        configure(ctx);
    }

    //Methods
    public StreamElement getContext() {
        return _Ctx;
    }
    private void setContext(StreamElement ctx) throws IllegalArgumentException {
        if (ctx == null)
            throw new IllegalArgumentException("context StreamElement cannot be null");
        _Ctx = ctx;
    }

    public Element write(StreamElement in) throws RuntimeException {
        StreamElement       parent;
        DocumentBuilder     b;
        Visitor             v;
        
        try {
            b = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            v = new Visitor(b.newDocument());
            parent = in.getParent();
            
            if (!(in instanceof StreamContext))
                in.setParent(getContext());

            in.accept(v);

            if (!(in instanceof StreamContext))
                in.setParent(parent);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        
        return v.getCurrent();
    }
    
    public void configure(StreamElement ctx) throws IllegalArgumentException, IllegalStateException, RuntimeException {
        if (getContext() != null)
            throw new IllegalStateException("This exporter is already configured");
        setContext(ctx);
    }
    public Object execute(StreamElement in) throws RuntimeException {
        return write(in);
    }
}