/*
 * InfoQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;

/**
 * <p>
 * Encapsulates an Info-Query (or iq) element.  The "iq" is typically used
 * in a "request/response" fashion, to get or set information.</p>
 *
 * <p>
 * An "iq" is the simplest <code>Packet</code>, relying on extensions for
 * its primary payload.  The possible types (currently) available to an
 * "iq" are:
 * <ul>
 * <li><code>{@link #GET}</code>:       Getting information.</li>
 * <li><code>{@link #SET}</code>:       Setting information.</li>
 * <li><code>{@link #RESULT}</code>:    The (successful) result of getting or
 * setting information.</li>
 * <li><code>{@link #ERROR}</code>:     The failed result of getting or
 * setting information.</li>
 * </ul></p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public interface InfoQuery extends Packet {
    //Inner-classes
    /**
     * <p>
     * Enumerates the possible types for <code>InfoQuery</code> objects.
     * In general, only the "static final" instances of this class should be
     * used.  "Custom" types can be defined by extending this class.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.2
     */
    public static class Type extends Packet.Type {
        //Constructors
        /**
         * <p>
         * Constructs a <code>Type</code> for the given a type name.</p>
         *
         * <p>
         * The value of <code>type</code> cannot be <code>null</code> or "", or
         * an <code>IllegalArgumentException</code> is thrown.</p>
         *
         * @param type Name of the type.
         * @since   JSO-0.2
         */
        protected Type(String type) {
            super(type);
        }
    }
    
    //"Constants"
    /** Represents the "get" type for <code>InfoQuery</code>.
     * @since   JSO-0.2
     */
    public static final Type GET = new Type("get");
    /** Represents the "set" type for <code>InfoQuery</code>.
     * @since   JSO-0.2
     */
    public static final Type SET = new Type("set");
    /** Represents the "result" type for <code>InfoQuery</code>.
     * @since   JSO-0.2
     */
    public static final Type RESULT = new Type("result");
}