/*
 * StreamFeatureset.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import java.util.List;

import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Interface representing the set of <tt>StreamFeature</tt> instances.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface StreamFeatureset extends Packet {
    //"Constants"
    public static final NSI     NAME = new NSI("features", Utilities.STREAM_NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the list of <tt>StreamFeature</tt> objects contained by this
     * <tt>StreamFeatureset</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is an instance of <tt>StreamFeature</tt>.</p>
     *
     * @return The list of contained <tt>StreamFeature</tt> instances.
     * @since   JSO-0.12
     */
    public List listFeatures();
    
    /**
     * <p>
     * Retrieves the first <tt>StreamFeature</tt> for the given namespace.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a matching
     * <tt>StreamFeature</tt> cannot be found.</p>
     *
     * <p>
     * The value of <tt>ns</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param ns The namespace URI to search for.
     * @return The <tt>StreamFeature</tt> for the given namespace.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public StreamFeature getFeature(String ns) throws IllegalArgumentException;
    /**
     * <p>
     * Retrieves the first <tt>StreamFeature</tt> implementing the given
     * interface.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tT> if a matching
     * <tt>StreamFeature</tt> cannot be found.</p>
     *
     * <p>
     * The value of <tt>iface</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. If <tt>iface</tt> is not
     * derived from <tt>StreamFeature</tt>, an <tt>IllegalArgumentException</tt>
     * is thrown.</p>
     *
     * @param iface The implemented interface to search for.
     * @return The <tt>StreamFeature</tt> for the given interface.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public StreamFeature getFeature(Class iface) throws IllegalArgumentException;
}