/*
 * MonitorListener.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.event;

/**
 * <p>
 * Interface defined to respond to <tt>PacketMonitor</tt> events.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface MonitorListener extends java.util.EventListener {
    //Methods
    /**
     * <p>
     * Method called when the <tt>PacketMonitor</tt> finds the expected
     * <tt>Packet</tt>.</p>
     *
     * <p>
     * The value of <tt>evt</tt> should never be <tt>null</tt>.</p>
     *
     * @param evt The <tt>MonitorEvent</tt> object.
     * @since   JSO-0.12
     */
    public void monitorFound(MonitorEvent evt);
    /**
     * <p>
     * Method called when the <tt>PacketMonitor</tt> times out.</p>
     *
     * <p>
     * The value of <tt>evt</tt> should never be <tt>null</tt>.</p>
     *
     * @param evt The <tt>MonitorEvent</tt> object.
     * @since   JSO-0.12
     */
    public void monitorTimeout(MonitorEvent evt);
    /**
     * <p>
     * Method called when the <tt>PacketMonitor</tt> fails due to an
     * exception.</p>
     *
     * <p>
     * The value of <tt>evt</tt> should never be <tt>null</tt>.</p>
     *
     * @param evt The <tt>MonitorEvent</tt> object.
     * @since   JSO-0.12
     */
    public void monitorFailed(MonitorEvent evt);
}