/*
 * DOMReader.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.io;

import org.jabberstudio.jso.StreamElement;

import org.w3c.dom.Element;

/**
 * <p>
 * Interface to generate <code>StreamElement</code> objects from W3C DOM
 * objects.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public interface DOMImporter extends Importer {
    //Methods
    /**
     * <p>
     * Generates a <code>StreamElement</code> from a W3C DOM element.</p>
     *
     * <p>
     * The import uses the <code>StreamContext</code> from {@link #getContext}
     * for any unknown namespace declarations.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.
     *
     * @param in The W3C DOM element to generate from.
     * @return The <code>StreamElement</code> for <code>in</code>
     * @throws RuntimeException If any errors occur generating the
     * <code>StreamElement</code>.
     * @since   JSO-0.2
     */
    public StreamElement read(Element in) throws RuntimeException;
}