/*
 * BindQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.core;


import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamFeature;

/**
 * <p>
 * Interface representing a resource binding query or stream feature.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface BindQuery extends Extension, StreamFeature {
    //"Constants"
    /** Namespace URI for <tt>BindQuery</tt> (urn:ietf:params:xml:ns:xmpp-bind).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE   = "urn:ietf:params:xml:ns:xmpp-bind";
    /** Qualified name for <tt>BindQuery</tt>
     * ({urn:ietf:params:xml:ns:xmpp-bind}bind).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME        = new NSI("bind", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the resource for this <tt>BindQuery</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The resource to bind.
     * @since   JSO-0.12
     */
    public String getResource();
    /**
     * <p>
     * Sets the resource for this <tt>BindQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>res</tt> may be <tt>null</tt> or "", in which case any
     * previous value is removed. If the value of <tt>res</tt> is not a valid
     * resource part as determined by {@link JID#isValidResource}, an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param res The new resource.
     * @throws IllegalArgumentException If the parameter is not valid.
     * @since   JSO-0.12
     */
    public void setResource(String res) throws IllegalArgumentException;

    /**
     * <p>
     * Retrieves the JID for this <tt>BindQuery</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The bound JID
     * @since   JSO-0.12
     */
    public JID getJID();
    /**
     * <p>
     * Sets the JID for this <tt>BindQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>jid</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param jid The new JID.
     * @since   JSO-0.12
     */
    public void setJID(JID jid);
}