/*
 * DelayedExtension.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.info;


import java.util.Date;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface for representing "jabber:x:delay" extensions, as documented in
 * JEP-0091.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href="http://www.jabber.org/jeps/jep-0091.html">JEP-0091: Delayed Delivery</a>
 */
public interface DelayedExtension extends Extension {
    //"Constants"
    /**
     * The namespace URI for <tt>DelayedExtension</tt> (jabber:x:delay).
     *
     * @since   JSO-0.12
     */
    public static final String      NAMESPACE   = "jabber:x:delay";
    /**
     * The qualified name for <tt>DelayedExtension</tt> ({jabber:x:delay}x).
     *
     * @since   JSO-0.12
     */
    public static final NSI         NAME        = new NSI("x", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the "from" for this <tt>DelayExtension</tt>. This value is
     * the JID from which the containing stanza originated.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has
     * not been set.</p>
     *
     * @return The "from" JID.
     * @since   JSO-0.12
     */
    public JID getFrom();
    /**
     * <p>
     * Sets the "from" for this <tt>DelayExtension</tt>.</p>
     *
     * <p>
     * The value of <tt>from</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param from The new "from" JID.
     * @since   JSO-0.12
     */
    public void setFrom(JID from);
    
    /**
     * <p>
     * Sets the timestamp for this <tt>DelayExtension</tt>. This value is the
     * date the stanza was originally sent.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if has
     * not been set.</p>
     *
     * @return The timestamp.
     * @since   JSO-0.12
     */
    public Date getStamp();
    /**
     * <p>
     * Sets the timestamp for this <tt>DelayExtension</tt>.</p>
     *
     * <p>
     * The value of <tt>stamp</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param stamp The new timestamp.
     * @since   JSO-0.12
     */
    public void setStamp(Date stamp);
}